# -*- coding: utf-8 -*-
"""

Analyzing convergence of MISMIP ensemble statistics as
a function of number of members
v1

"""


import os
import numpy as np
import netCDF4 as nc
import matplotlib.pyplot as plt
from scipy import stats

nruns           = 500 #nb of ensemble members
nsteps          = 500 #nb of time steps
nmembsplot      = np.arange(10,501,5)

DirResults         = os.path.dirname(os.path.realpath(__file__))+'/'

rholevels  = np.array([0.0,0.6,0.9,0.95,0.98])
pathsRes   = [DirResults+'NCfileMISMIP_StochMelt_547Ens_v1.nc',
              DirResults+'NCfileMISMIP_RunARonlyMeltAc6_546Ens_v1.nc',
              DirResults+'NCfileMISMIP_RunARonlyMeltAc9_547Ens_v1.nc',
              DirResults+'NCfileMISMIP_RunARonlyMeltAc95_546Ens_v1.nc',
              DirResults+'NCfileMISMIP_RunARonlyMeltAc98_546Ens_v1.nc']
taulevels  = np.round(1/(1-rholevels),1)

ls_colors   = plt.cm.rainbow(np.linspace(0,1,len(taulevels)))
ls_labels = ['$MISMIP_{\u03C4=1}$',
             '$MISMIP_{\u03C4=2.5}$',
             '$MISMIP_{\u03C4=10}$',
             '$MISMIP_{\u03C4=20}$',
             '$MISMIP_{\u03C4=50}$']

### Random selection of indices ###
isel = []
for nn,num in enumerate(nmembsplot):
    if(nn==0):
        isel.append(np.random.randint(0,nruns,size=num))
    else:
        allinds  = np.random.randint(0,nruns,nruns).tolist() #all indices in random order
        numextra = num-nmembsplot[nn-1]
        indsnew  = [ind for ind in allinds if((ind in isel[nn-1])==False)][0:numextra]
        isel.append(np.append(isel[nn-1],indsnew))

for ii,tauval in enumerate(taulevels):
    ds = nc.Dataset(pathsRes[ii])

    ### Load ###
    timear         = np.array(ds.variables['time'])
    if(ii==0):
        ### Prepare arrays ###
        MnFullMvals      = np.zeros((len(taulevels),len(nmembsplot),len(timear)))
        SdevFullMvals    = np.zeros((len(taulevels),len(nmembsplot),len(timear)))
        SkewFullMvals    = np.zeros((len(taulevels),len(nmembsplot),len(timear)))
        MnSelMvals       = np.zeros((len(taulevels),len(nmembsplot),len(timear)))
        SdevSelMvals     = np.zeros((len(taulevels),len(nmembsplot),len(timear)))
        SkewSelMvals     = np.zeros((len(taulevels),len(nmembsplot),len(timear)))

    fullmatMvals       = 917*1e-12*np.array(ds.variables['volume'])[0:nruns,:] #[Gt]
    ds.close()
    ### Stats on the full ensemble ###
    MnFullMvals[ii,:,:]         = np.mean(fullmatMvals,axis=0)
    SdevFullMvals[ii,:,:]       = np.std(fullmatMvals,axis=0)
    SkewFullMvals[ii,:,:]       = stats.skew(fullmatMvals,axis=0)
    ### Stats on the random selections ###
    for jj in range(len(nmembsplot)):
        selmatMvals             = fullmatMvals[isel[jj],:]
        MnSelMvals[ii,jj,:]     = np.mean(selmatMvals,axis=0)
        SdevSelMvals[ii,jj,:]   = np.std(selmatMvals,axis=0)
        SkewSelMvals[ii,jj,:]   = stats.skew(selmatMvals,axis=0)


### Take relative values to the full ensemble values ###
relMnSelMvals    = MnSelMvals/MnFullMvals
relSdevSelMvals  = SdevSelMvals/SdevFullMvals


### Plotting ###
fig = plt.figure(figsize=[11,10])
ax  = plt.subplot(311)
for rr,taulevel in enumerate(taulevels):
    ax.plot(nmembsplot,MnSelMvals[rr,:,-1],c=ls_colors[rr],linewidth=1.0,marker='o',markersize=4,label=ls_labels[rr])
ax.tick_params(axis='y', which='major', labelsize=14)
ax.set_ylabel('Mean final ice mass [Gt]',fontsize=14)
ax.tick_params(axis='y', which='major',labelsize=14)
ax.tick_params(axis='x', which='major',labelcolor='w')
ax.text(-10,np.max(MnSelMvals[:,:,-1]-30),'(a)',fontsize=14)
ax  = plt.subplot(312)
for rr,taulevel in enumerate(taulevels):
    #ax.plot(nmembsplot,SdevSelMvals[rr,:,-1],c=ls_colors[rr],linewidth=1.0,marker='o',markersize=4,label=ls_labels[rr])
    ax.plot(nmembsplot,relSdevSelMvals[rr,:,-1],c=ls_colors[rr],linewidth=1.0,marker='o',markersize=4,label=ls_labels[rr])
ax.tick_params(axis='y', which='major', labelsize=14)
ax.set_ylabel(r'$\dfrac{\sigma_{E}}{\sigma_{E,500}}$ in final ice mass',fontsize=14)
ax.tick_params(axis='y', which='major',labelsize=14)
ax.tick_params(axis='x', which='major',labelcolor='w')
ax.text(-10,np.max(relSdevSelMvals[:,:,-1]-0.01),'(b)',fontsize=14)
ax  = plt.subplot(313)
for rr,taulevel in enumerate(taulevels):
    ax.plot(nmembsplot,SkewSelMvals[rr,:,-1],c=ls_colors[rr],linewidth=1.0,marker='o',markersize=4,label=ls_labels[rr])
ax.tick_params(axis='y', which='major', labelsize=14)
ax.set_ylabel('Skew in final ice mass',fontsize=14)
ax.tick_params(axis='y', which='major',labelsize=14)
ax.tick_params(axis='x', which='major',labelsize=14)
ax.text(-10,np.max(SkewSelMvals[:,:,-1]-0.04),'(c)',fontsize=14)
ax.set_xlabel('Number of ensemble members',fontsize=14)
ax.legend(loc='best',fontsize=14,ncol=3)
fig.tight_layout()


